package Ecodes;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import java.io.*;

public class ECodes
    extends MIDlet
    implements CommandListener {

  private final static Command CMD_EXIT = new Command("Exit", Command.EXIT, 1);
  private final static Command CMD_OK = new Command("OK", Command.OK, 1);
  private final static Command CMD_BACK = new Command("Back", Command.BACK, 1);

  private final static int DB_SIZE = 28122;
  private final static int MAX_RESULTS = 10;

  private final static String[] defaultList = {
      "No results."};

  private Form detailForm, waitForm;
  private TextBox searchForm;
  private List resultForm;

  private StringItem waitStringItem, codeStringItem, nameStringItem,
      descStringItem, useStringItem;

  private char[] dbChar = new char[DB_SIZE + 1];
  private String db;

  private int numResults;
  private int[] posCode = new int[MAX_RESULTS];
  private int[] posName = new int[MAX_RESULTS];
  private int[] posDesc = new int[MAX_RESULTS];
  private int[] posUse = new int[MAX_RESULTS];
  private int[] posEnd = new int[MAX_RESULTS];

  public ECodes() {
    // create forms
    searchForm = new TextBox("Search for E code:", null, 4,
                             TextField.NUMERIC /*0x80000*/);
    resultForm = new List("Results", List.IMPLICIT, defaultList, null);
    detailForm = new Form("Detail");
    waitForm = new Form("Busy");
  }

  private void setupForms() {
    // Wait form
    waitStringItem = new StringItem("Please wait", "Loading database ...");
    waitForm.append(waitStringItem);
    waitForm.addCommand(CMD_EXIT);
    waitForm.setCommandListener(this);
    // Search Form
    searchForm.addCommand(CMD_OK);
    searchForm.addCommand(CMD_EXIT);
    searchForm.setCommandListener(this);
    // Result Form
    resultForm.addCommand(CMD_BACK);
    resultForm.addCommand(CMD_EXIT);
    resultForm.setCommandListener(this);
    // Detail Form
    codeStringItem = new StringItem("Code", null);
    nameStringItem = new StringItem("Name", null);
    descStringItem = new StringItem("Description", null);
    useStringItem = new StringItem("Usage", null);
    detailForm.append(codeStringItem);
    detailForm.append(nameStringItem);
    detailForm.append(descStringItem);
    detailForm.append(useStringItem);
    detailForm.addCommand(CMD_BACK);
    detailForm.addCommand(CMD_EXIT);
    detailForm.setCommandListener(this);
  }

  private boolean loadData() {
    InputStreamReader isr = new InputStreamReader(getClass().
                                                  getResourceAsStream(
        "data.bin"));
    try {
      isr.read(dbChar, 0, DB_SIZE);
      isr.close();
      db = new String(dbChar);
    }
    catch (IOException e) {
      return false;
    }
    return true;
  }

  public void doSearch(String s) {
    waitStringItem.setText("Searching ...");
    Display.getDisplay(this).setCurrent(waitForm);

    numResults = 0;
    while (resultForm.size() > 0) {
      resultForm.delete(0);
    }
    int p = -1;

    while ( (p = db.indexOf(s, p + 1)) >= 0 &&
           (numResults < MAX_RESULTS)) {
      // add element to the list
      int c = numResults;
      posCode[c] = db.lastIndexOf('\n', p) + 1;
      posName[c] = db.indexOf(';', posCode[c]) + 1;
      posDesc[c] = db.indexOf(';', posName[c]) + 1;
      posUse[c] = db.indexOf(';', posDesc[c]) + 1;
      posEnd[c] = db.indexOf('\n', posUse[c]) + 1;
      if (posEnd[numResults] < 0) {
        posEnd[c] = db.length();
      }
      String item = new String(db.substring(posCode[c], posName[c] - 1));
      item = item.concat(": ").concat(db.substring(posName[c], posDesc[c] - 1));
      resultForm.append(item, null);
      numResults++;
    }
    // display the list, if not empty, otherwise the default one
    if (numResults == 0) {
      resultForm.append(defaultList[0], null);
    }
  }

  private void displayResult(int c) {
    codeStringItem.setText(db.substring(posCode[c], posName[c] - 1));
    nameStringItem.setText(db.substring(posName[c], posDesc[c] - 1));
    descStringItem.setText(db.substring(posDesc[c], posUse[c] - 1));
    useStringItem.setText(db.substring(posUse[c], posEnd[c] - 1));
  }

  protected void startApp() {
    // setup forms
    setupForms();
    Display.getDisplay(this).setCurrent(waitForm);
    // load data
    if (loadData()) {
      Display.getDisplay(this).setCurrent(searchForm);
    }
    else {
      waitStringItem.setText("Error loading database");
    }
  }

  protected void destroyApp(boolean unconditional) {
  }

  protected void pauseApp() {
  }

  public void commandAction(Command c, Displayable d) {
    if (c == CMD_EXIT) {
      destroyApp(false);
      notifyDestroyed();
    }
    if (c == CMD_OK) {
      if (searchForm.getString().length() > 0) {
        doSearch(searchForm.getString());
        Display.getDisplay(this).setCurrent(resultForm);
      }
    }
    if (c == CMD_BACK) {
      if (resultForm.isShown()) {
        Display.getDisplay(this).setCurrent(searchForm);
      }
      if (detailForm.isShown()) {
        Display.getDisplay(this).setCurrent(resultForm);
      }
    }
    if (c == List.SELECT_COMMAND) {
      if (numResults > 0) {
        displayResult(resultForm.getSelectedIndex());
        Display.getDisplay(this).setCurrent(detailForm);
      }
    }
  }

}
